#!/usr/bin/env python

"""Tests for `salary_stone` package."""

import pytest

from salary_stone.skill_extractor import Skill_Extractor

@pytest.fixture
def prepare():
    SkillE = Skill_Extractor()
    jobdesc = """
    python is a great skill as is azure and it's also great that we can use data modeling to answer some questions.
    """
    yield SkillE, jobdesc
    
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


class TestSkill:

    def test_default(self, prepare):
        SkillE = prepare[0]
        jobdesc = prepare[1]
        result = SkillE.extract_skills(jobdesc)
        assert len(result) == 3
        assert set(result) == set(['python', 'azure', 'data modeling'])
    
    def test_empty(self, prepare):
        SkillE = prepare[0]
        result = SkillE.extract_skills('')
        assert len(result) == 0

    def test_error(self, prepare):
        SkillE = prepare[0]
        with pytest.raises(Exception):
            SkillE.extract_skills(123)
    
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string



import pytest
from salary_stone.salary_extractor import Salary_Extractor

@pytest.fixture
def prepare():
    SalaryE = Salary_Extractor()
    jobdesc = """
    python is a great skill as is azure and it's also great that we can use data modeling to answer some questions.
    """
    yield SalaryE, jobdesc
    
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')

class TestSalary:
    def test_default(self, prepare):
        SalaryE = prepare[0]
        jobdesc = prepare[1]
        result = SalaryE.extract_salary(jobdesc)
        assert type(result) == list
        assert len(result) == 2
        assert type(result[0]) == float
        assert result[0] < result[1]
    
    def test_empty(self, prepare):
        SalaryE = prepare[0]
        result = SalaryE.extract_salary('')
        assert result == None

    def test_no_skill(self, prepare):
        SalaryE = prepare[0]
        result = SalaryE.extract_salary('There are no skills here to extract.')
        assert result == None
    
    def test_error(self, prepare):
        SalaryE = prepare[0]
        with pytest.raises(Exception):
            SalaryE.extract_salary(1)


"""Main module."""
import spacy
import os
#Data loading/ Data manipulation

#nltk
import re
import nltk
from nltk.corpus import stopwords
from nltk.stem import WordNetLemmatizer

import ssl

try:
    _create_unverified_https_context = ssl._create_unverified_context
except AttributeError:
    pass
else:
    ssl._create_default_https_context = _create_unverified_https_context
    
nltk.download(['stopwords','wordnet', 'omw-1.4'])

fp = str(os.path.dirname(__file__))
class Skill_Extractor:
    """
    This class contains the methods related to extracting the skills from text. The class takes in a .jsonl file which has
    all of the skills available for extraction and a model that is a spacy model which has also been installed.
    """
    def __init__(self, model:str ='en_core_web_sm', skill_dict:str =fp+'/data/jz_skill_patterns.jsonl'):
        self.nlp = spacy.load(model)
        ruler = self.nlp.add_pipe("entity_ruler")
        ruler.from_disk(skill_dict)
    
    def extract_skills(self, text: str):
        """
        The purpose of this method is to clean the text data being input and return the unique
        set of skills found within the string.
        
        :param: text is the text where the skills are being extracted from.
        
        :returns: the unique list of skills
        """
        review = re.sub(
            '(@[A-Za-z0-9]+)|([^0-9A-Za-z \t])|(\w+:\/\/\S+)|^rt|http.+?"',
            " ",
            text,
        )
        review = review.lower()
        review = review.split()
        lm = WordNetLemmatizer()
        review = [
            lm.lemmatize(word)
            for word in review
            if not word in set(stopwords.words("english"))
        ]
        review = " ".join(review)
        skills = self.get_skills(review.lower())
        skills_set = list(set(skills))
        return(skills_set)
    
    def get_skills(self, text:str):
        """
        The purpose of this method is to extract all the skills from the text.
        """
        doc = self.nlp(text)
        myset = []
        subset = []
        for ent in doc.ents:
            if ent.label_ == "SKILL":
                subset.append(ent.text)
        myset.append(subset)
        return subset


import pickle
import pandas as pd
import os
from .skill_extractor import Skill_Extractor

fp = str(os.path.dirname(__file__))
class Salary_Extractor:
    """
    The purpose of this class is to extract out salary based on skills.
    """
    def __init__(self, model=fp+'/data/model.pkl', skill_extractor=Skill_Extractor(), sp=['ai',
 'algorithm',
 'analytics',
 'artificial intelligence',
 'azure',
 'big data',
 'business',
 'business intelligence',
 'collaboration',
 'computer science',
 'data analysis',
 'data management',
 'data mining',
 'data model',
 'data modeling',
 'data quality',
 'data science',
 'data visualization',
 'data warehouse',
 'database',
 'deep learning',
 'deployment',
 'design',
 'documentation',
 'engineering',
 'finance',
 'framework',
 'hadoop',
 'library',
 'machine learning',
 'marketing',
 'ml',
 'monitoring',
 'play',
 'programming language',
 'project management',
 'python',
 'relational database',
 'scala',
 'security',
 'server',
 'software',
 'support',
 'tableau',
 'tensorflow',
 'testing',
 'visualization',
 'workflow']):
        self.skill_extractor = skill_extractor
        self.sp = sp
        with open(model, 'rb') as f:
            self.mlb, self.clf = pickle.load(f)
            
    
    def extract_salary(self, text: str):
        skill_vec = self.skill_extractor.extract_skills(text)
        if len(skill_vec) == 0:
            return None
        tmp = pd.DataFrame({'skills': [skill_vec]})

        tmp = tmp.join(
                    pd.DataFrame.sparse.from_spmatrix(
                        self.mlb.transform(tmp.pop('skills')),
                        index=tmp.index,
                        columns=self.mlb.classes_))
        res = str(self.clf.predict(tmp[self.sp].to_numpy())[0])
        res = res.split('-')
        res = [float(x) for x in res] 
        return res